@@ -5,6 +5,7 @@ use Test::More;
 use File::Glob qw(bsd_glob);
 use Config '%Config';
 use File::Spec;
+use Time::HiRes qw( time sleep );
 use Carp qw(croak);
 
 delete $ENV{HTTP_PROXY};
@@ -24,7 +25,7 @@ sub browser_instances {
 
     # add author tests with local versions
     my $spec = $ENV{TEST_WWW_MECHANIZE_PHANTOMJS_VERSIONS}
-             || 'phantomjs-versions/*/{*/,}phantomjs*'; # sorry, likely a bad default
+             || 'phantomjs-versions/*/{*/,}phantomjs*$Config{_exe}'; # sorry, likely a bad default
     push @instances, sort {$a cmp $b} grep { -x } bsd_glob $spec;
 
     # Consider filtering for unsupported PhantomJS versions here
@@ -44,12 +45,13 @@ sub run_across_instances {
 
     for my $browser_instance (@$instances) {
         if ($browser_instance) {
-            diag sprintf "Testing with %s",
+            note sprintf "Testing with %s",
                 $browser_instance;
         };
         my @launch = $browser_instance
                    ? ( launch_exe => $browser_instance,
-                       port => $port )
+                       #port => $port
+                     )
                    : ();
 
         my $mech = eval { $new_mech->(@launch) };
@@ -63,21 +65,26 @@ sub run_across_instances {
                     launch_exe => $browser_instance
                 });
             };
-            diag sprintf "PhantomJS version '%s'", $version;
+            note sprintf "PhantomJS version '%s'", $version;
             next
         };
 
-        diag sprintf "PhantomJS version '%s', ghostdriver version '%s'",
+        note sprintf "PhantomJS version '%s', ghostdriver version '%s'",
             $mech->phantomjs_version, $mech->ghostdriver_version;
 
         # Run the user-supplied tests
         $code->($browser_instance, $mech);
 
         # Quit in 500ms, so we have time to shut our socket down
+        my $pid = $mech->{pid};
         undef $mech;
-        sleep 2; # So the browser can shut down before we try to connect
+
+        my $timeout = time + 2;
+        while( kill 0 => $pid and time < $timeout ) {
+            sleep 0.1; # So the browser can shut down before we try to connect
+        };
         # to the new instance
     };
 };
 
-1;
\ No newline at end of file
+1;
